%% MANIA - Machine learning Application for NeuroImaging Analyses
% author: Dominik Grotegerd
% Neuroimaging Group @University Hospital Münster
% License: BSD
%
% Commands:
%   mania                  - starts MANIA Graphical User Interface
%   mania('version')       - returns actual version number
%   mania('license')       - returns licence
%
% Version Changes          -> changelog.txt
% Documentation            -> Directory: doc
% third party software     -> Directory: extlib
function varargout = mania(varargin)

% Constants
versionNr = '2.5.1';
versionYear = '2013';
author = 'Dominik Grotegerd';
institute = 'Neuroimaging Group, Dep. of Psychiatry, University Hospital Muenster, Germany';
groupURL = 'http://www.uni-muenster.de/neuroimage';


[mania_path ign] = fileparts(which('mania'));
% setup path & launch GUI
if isempty(varargin)
    addpath(mania_path);
    addpath([mania_path filesep 'oo']);
    addpath([mania_path filesep 'gui']);
    addpath([mania_path filesep 'util']);
    addpath(genpath([mania_path filesep 'extlib']));
    mania_gui_main
% return infos
elseif isstr(varargin{1})
    switch varargin{1}
        case 'version'
            fprintf([mania('getVersionString') '\n']);
        case 'getVersionString'
            varargout{1} = ['MANIA version ' versionNr ' (' versionYear ')'];
        case 'license'
            lic = mania('getLicenseString');
            cellfun(@(x) fprintf([x '\n']),lic{:});
        case 'getLicenseString'
            fid = fopen([mania_path filesep 'license.txt']);
            varargout{1} = textscan(fid, '%s','delimiter', '\n');
            fclose(fid);
        case 'getAuthor'
            varargout{1} = author;
        case 'group'
            varargout{1} = institute;
        case 'www'
            varargout{1} = groupURL;
        otherwise
            error('Unkown option');
    end
else
    error('Unkown parameter');
end




