classdef mania_ensembleFitter < mania_classifier & mania_statisticsWrapper
    % Ensemble Fitter: uses "fitensemble"-function of statistics toolbox
    % in general: ensemble methods of classification trees
    % look at >help fitsensemble for more information
    
    properties (SetAccess = protected)
        treeAmount  = 300;              % how many trees
        learners    = 'Tree';           % learner type
        method      = 'AdaBoostM1';     % classification method
    end
    
    properties (Constant)
        ensembleFunctions = {'AdaBoostM1', 'LogitBoost', 'GentleBoost', 'RobustBoost','LPBoost','RUSBoost','TotalBoost','Subspace','Bag','AdaBoost2'}; % available methods
    end
    
    methods (Static)
        function shortName = name
            % name, that appears in GUI
            shortName = 'Ensemble Fitter';
        end
    end
    
    methods
        
        function this = train(this,trainPatterns,trainLabels)
            % train with training data and according labels
            this.classifierStruct = fitensemble(trainPatterns',trainLabels, this.method,this.treeAmount,this.learners,'type','classification');
        end
        
        function [predictions decFactors] = test(this,testPatterns)
            % predict data
            output = this.classifierStruct.predict(testPatterns');
            predictions = [];
            for k=1:size(output,1)
                predictions = [predictions output(k)];
            end
            % not implemented yet!
            decFactors = struct;
        end
        
        function classifierStruct  = getClassifier(jc,trainPatterns,trainLabels)
            % Not implemented yet
            error('Not implemented');
        end
        
        function jc = uiconfigure(jc)
            % simple minimum gui prompt to configure the classifier
            mthdIdx = listdlg('PromptString','Select Ensemble Method:','SelectionMode','single','ListString',jc.ensembleFunctions);
            if ~isempty(mthdIdx)
                jc.method = jc.ensembleFunctions{mthdIdx};
            end
            repl=inputdlg({'learning cycles',},'Settings',1,{''});
            if ~isempty(repl{1})
                jc.treeAmount = str2double(repl{1});
            end
        end
        
        function new = copy(this)
            new = copy@mania_classifier(this);
            new.treeAmount  = this.treeAmount;
            new.learners    = this.learners;
            new.method      = this.method;
        end
        
        function show(jc)
            show@mania_classifier(jc);
            fprintf('#Trees:\t\t%i\n', jc.treeAmount);
            fprintf('Learner type:\t%s\n', jc.learners);
            fprintf('Method:\t\t%s\n', jc.method);
        end
        
    end
    
    
end