classdef mania_fs_average < mania_fs
    % Dimensionality Reduction: Average Image from a list of images
    % if 4-Dimensional images as input:
    %     every 3-Dimensional Image is taken into account - output will in
    %     every case be a 3-Dimensional image (unless you change dimToCat).
    
   properties 
      name = 'Average';           
      dimToCat = 4;
   end
   
   methods 
       function avg = reduce(fs,varargin)
           % returns one averaged image from multiple input
          if nargin > 1
               avg = varargin{1};
               if ~isnumeric(avg)
                       throw(MException('mania:WrongImageFormat','Wrong image format for this method'));
                   else
                   if nargin > 2
                       append = mania_fs_append;
                       avg = append.reduce(varargin{:});
                   end
                   if ndims(avg) == fs.dimToCat
                       volSize = size(avg);
                       avg = sum(avg, fs.dimToCat);
                       avg = avg/volSize(fs.dimToCat);
                   else
                       warning('mania:tooLessImgs','There was only one image!');
                   end
               end
          else
                warning('mania:emptyImg','Image was empty!')
                avg = [];
          end
       end
   end
    
end