% This is just a wrapper to the SVM of the Matlab Bioinformatics Toolbox
classdef mania_matlabSVM < mania_multiClassWrapper & mania_bioinfoWrapper
    
    properties (SetAccess = protected)
        svmParams = {};             % SVM Options
        svmOpts;                    % svmOpts - see "help svmOpts"
    end
    
    properties (Constant)
        availableKernelFunctions = { ...
            'linear',...
            'quadratic',...
            'polynomial',...
            'rbf',...
            'mlp'  };           % available Kernel Functions
    end
    
    properties
        multiClassMethod = 'onevsone';          % Matlab SVM only returns class labes, therfore only one-vs-one is possible
    end
    
    methods (Static)
        function shortName = name
            shortName = 'Support Vector Machine (Bio-Info-Toolbox)';
        end
    end
    
    methods
        
        function jc = binaryTrain(jc,trainPatterns,trainLabels)
            % binary training using svmtrain
            
            % since old method does not work anymore: switch to
            % biolearning directory and switch back afterwards... -_-
            current = pwd;
            try cd([matlabroot filesep 'toolbox' filesep 'bioinfo' filesep 'biolearning' filesep]);
                jc.classifierStruct = svmtrain(trainPatterns,trainLabels, jc.svmParams{:});
                cd(current);
            catch exp
                cd(current);
                rethrow(exp)
            end
        end
        
        function [predictions decFactors] = binaryTest(jc,testPatterns)
            % just executes svmclassify
            predictions = svmclassify(jc.classifierStruct,testPatterns');
            % not implemented yet!
            decFactors = struct;
        end
        
        function jc = setSVMparams(jc, varargin)
            % Set additional Params of the Matlab SVM
            jc.svmParams = varargin;
        end
        
        function classifierStruct  = getClassifier(jc,trainPatterns,trainLabels)
            % switch to biolearning directory.
            current = pwd;
            try
                cd([matlabroot filesep 'toolbox' filesep 'bioinfo' filesep 'biolearning' filesep]);
                classifierStruct = svmtrain(trainPatterns,trainLabels, jc.svmParams{:});
                cd(current);
            catch exp
                cd(current)
                rethrow(exp)
            end
        end
        
        function jc = uiconfigure(jc)
            jc.svmParams = {};
            kernelIdx = listdlg('PromptString','Select Kernel Function:','SelectionMode','single','ListString',jc.availableKernelFunctions);
            if ~isempty(kernelIdx)
                jc.svmParams{length(jc.svmParams)+1} = 'KERNEL_FUNCTION';
                jc.svmParams{length(jc.svmParams)+1} = jc.availableKernelFunctions{kernelIdx};
            end
            repl=inputdlg({'BoxConstraint Value','Method (QP,SMO,LS)','Tolerance KKT','MaxIter','KKT Violation Lvl.','Kernel Cache Limit'},'SVM Settings',1,{'','','','','',''});
            if ~isempty(repl{1})
                jc.svmParams{length(jc.svmParams)+1} = 'BOXCONSTRAINT';
                jc.svmParams{length(jc.svmParams)+1} = str2double(repl{1});
            end
            if ~isempty(repl{2})
                jc.svmParams{length(jc.svmParams)+1} = 'METHOD';
                jc.svmParams{length(jc.svmParams)+1} = repl{2};
            end
            % if there are svmoset-options
            if ~isempty(horzcat(repl{3:end}))
                addOpts = {};
                if ~isempty(repl{3})
                    addOpts{length(addOpts)+1} = 'TolKKT';
                    addOpts{length(addOpts)+1} = str2double(repl{3});
                end
                if ~isempty(repl{4})
                    addOpts{length(addOpts)+1} = 'MaxIter';
                    addOpts{length(addOpts)+1} = str2double(repl{4});
                end
                if ~isempty(repl{5})
                    addOpts{length(addOpts)+1} = 'KKTViolationLevel';
                    addOpts{length(addOpts)+1} = str2double(repl{5});
                end
                if ~isempty(repl{6})
                    addOpts{length(addOpts)+1} = 'KernelCacheLimit';
                    addOpts{length(addOpts)+1} = str2double(repl{6});
                end
                jc.svmOpts = svmsmoset(addOpts{:});
                jc.svmParams{length(jc.svmParams)+1} = 'SMO_OPTS';
                jc.svmParams{length(jc.svmParams)+1} = jc.svmOpts;
            end
        end
        
        function show(jc)
            show@mania_multiClassWrapper(jc);
            for k=1:2:length(jc.svmParams)
                if isstruct(jc.svmParams{k})
                    fprintf('---------------------\nsvmsoset:\n')
                    jc.svmParams{k}
                elseif isnumeric(jc.svmParams{k+1})
                    fprintf('%s      - %5.0f\n', jc.svmParams{k}, jc.svmParams{k+1});
                else
                    fprintf('%s      - %s\n', jc.svmParams{k}, jc.svmParams{k+1});
                end
            end
        end
        
        function new = copy(this)
            new = copy@mania_classifier(this);
            new.svmParams  = this.svmParams;
            new.svmOpts = this.svmOpts;
        end
        
        function new = binaryCopy(this)
            new = mania_matlabSVM;
            new.svmParams = this.svmParams;
            new.svmOpts = this.svmOpts;
            new.isBinary = 1;
        end
        
    end
end