% Interface for all methods, that somehow change/reduce set of features
classdef mania_patternManipulatorMethod < handle
    
    properties (Access = protected)
        serviceMethods                  % helper class, if information from original images are needed (e.g. resize mask files)
    end
    
    
    methods (Static)
        function bool = isConfigurable(this)
            % return true, if "uiconfigure" is needed / available
            bool = false; 
        end
        
        function [state msg] = checkSetup
        % return true, if all dependencies are given, false if something is
        % missing
            state = true;
            msg = 'Ok.';
        end

        function beVisible = shouldBeVisible
            % should method appear in list of choosable methods?
            beVisible = true;
        end
    end
    
    methods (Abstract)
        % apply on patterns
        [train, test]           = operate(this,train, test, trainLabels);     
        % reconstruct patterns
        patternReconstructed    = reconstruct(this,patternIn);                
        % apply the changes also to further, independent images (like a roi)
        patternOut              = reoperate(this,patternIn);                  
    end
    
    methods (Abstract, Static)
        % human readable name, that appears in GUI
        shortName               = name;           
        % deprecated
        desc                    = description;
    end
    
    methods
        function new = copy(this)
            % return copy of this object
            new = feval(class(this));
            new.serviceMethods = this.serviceMethods;
        end
        
        function setServiceMethods(this,serviceMethods)
            % set service Methods (see mania_patternServiceMethods)
           this.serviceMethods = serviceMethods; 
        end
        
        function viewSettings(this)
            % human readable setup of the method
            fprintf('\nShort Description:\n:::      "%s" :::\n\n',this.description);
            if ~this.isConfigurable
                fprintf(' No settings available.\n');
            else
                fprintf(' Settings:\n')
            end
        end
        
    end
    
end