% Applies before crossvalidation a mask, that filter NANs out of the
% patterns
% In detail:
%   For k = 1 to numberOfVoxels
%       look at every subject and check this k-th voxel
%       if subject is found, where voxel is NAN: Mark this voxel in a mask
%   end
%   Now apply this mask to every subject.
classdef mania_pre_NANmask < mania_nan_handling
    
    properties (SetAccess = protected)
        mask;               % mask, that should be applied
    end
    
    methods (Static)
        function desc = description 
            desc = 'Mask NaNs: uses a mask to shadow all voxels, which have no value in one pattern.';
        end
        function shortName = name
            shortName = 'Mask NaNs';
        end
    end

    methods
        
        function patternOut = operate(this, patternIn)
            % Search for NANs and mask them in every pattern
            nanFounds = any(isnan(patternIn),2);
            this.mask = ~nanFounds;
            patternOut = patternIn(this.mask,:);
        end
        
        function patternOut = reoperate(this, patternIn)
            % replicate actions on another pattern
            %   in general this could be a roi, that later also should be
            %   applied on the patterns.
            patternIn = reshape(patternIn,numel(patternIn),1);
            if size(patternIn,1) < size(this.mask,1) && ~mod(size(this.mask,1),size(patternIn,1))
                for k=1:size(this.mask,1)/size(patternIn,1)
                    patternIn = [patternIn; patternIn];
                end
            end
            patternOut = patternIn(this.mask,:);
        end
        
        function patternOut = reconstruct(this, patternIn)
            % Reconstruct a pattern to it original dimension, therefore set NaNs to zero.
            % This is valid, because reconstructing is done in the context of
            % discriminative maps, which reflect the influence of voxel on the
            % accuracy. When a voxel is zero, this means, there is no
            % influence.
            patternOut = zeros(size(this.mask,1),1);
            patternOut(this.mask) = patternIn;
        end
        
        function viewSettings(this)
            viewSettings@mania_preprocessor(this);
            if ~isempty(this.mask)
                fprintf('  voxels included:  %i\n',sum(~reshape(this.mask,numel(this.mask),1)));
            end
        end
        
    end
end