% Imputation of missing values using kknimpute (Bioinformatics toolbox)
classdef mania_pre_knnimpute < mania_nan_handling & mania_bioinfoWrapper
    
    properties (Constant)
        availableDistances = {  'euclidean'; ...
            'cityblock'; ...
            'cosine'; ...
            'correlation'; ...
            'Hamming' };
    end
    
    properties (SetAccess = protected)
        k = 5; % k for nearest neighbours
        distFunc = 'euclidean';
    end
    
    methods (Static)
        function desc = description(this)
            desc = 'Impute unkown values by using knnimpute (Matlab function)';
        end
        function shortName = name
            shortName = 'Imputation by KNN';
        end
    end
    
    methods
        function bool = isConfigurable(preprop)
            bool = true;
        end
        
        function patternOut = operate(this, patternIn)
            % Search for NANs and mask them in every pattern
            patternOut = knnimpute(patternIn,this.k,'Distance',this.distFunc);
        end
        
        function patternOut = reoperate(this, patternIn)
            if any(isnan(patternIn))
                warning('Not implemented! NANs could not be removed.');
            end
            patternOut = patternIn;
        end
        
        function patternOut = reconstruct(this, patternIn)
            % Nothing to do here, because no voxels were removed.
            patternOut = patternIn;
        end
        
        function this = uiconfigure(this)
            kStr=inputdlg('k for k-nearest...','The K',1,{num2str(this.k)});
            if ~isempty(str2num(kStr{:})) && rem(str2num(kStr{:}),1) == 0
                this.k = str2num(kStr{:});
                dfIdx = listdlg('PromptString','Select Disctance Function:','SelectionMode','single','ListString',this.availableDistances);
                disFct = this.availableDistances(dfIdx);
                this.distFunc = disFct{:};
            else
                error('Not a valid value');
            end
        end
        
        function viewSettings(this)
            viewSettings@mania_preprocessor(this);
            fprintf('k    =     %d\n', this.k);
            fprintf('Distance\n Func.:    %s', this.distFunc);
        end
        
        function new = copy(this)
            new = copy@mania_preprocessor(this);
            new.k = this.k;
            new.distFunc = this.distFunc;
        end
        
    end
end