% according to Wang et al. 2003:
% 
% compute average / min. / max. values of rois for each subject as new
% pattern representation
classdef mania_pre_range_rois < mania_pre_active_rois
    
    properties (SetAccess = protected)
        minMeanMax = [0 0 0];           % logical index if min/mean/ and/or max. values should be computed
    end
    
    methods (Static)
        function desc = description
            desc = 'Range of ROIs: min, max and/or average value in a ROI';
        end
        function shortName = name
            shortName = 'Min/Max/Avg of ROIs';
        end
        function beVisible = shouldBeVisible
            beVisible = true;
        end
    end
    
    methods
        
        function patternOut = operate(this, patternIn)
            if isempty(this.regions)
                throw(MException('mania:MaskWasNotSet', [this.name ': No mask was specified!']));
            end
            for k=1:length(this.regions)
                this.regions{k}.previousOperations = this.previousOperations;
            end
            patCount = size(patternIn,1)/numel(this.regions{1}.mask);
            if (patCount > 1)
                patSize = size(patternIn,1)/patCount;
                activn = [];
                for h=1:patCount
                    activn = [activn; this.operate(patternIn(patSize*(h-1)+1:patSize*h,:))];
                end
            else
                activn = zeros(length(this.regions)*sum(this.minMeanMax),size(patternIn,2));
                % 4 every roi:
                for k=1:length(this.regions)
                    this.regions{k}.setServiceMethods(this.serviceMethods);
                    for sub=1:size(patternIn,2)
                        temproi = this.regions{k}.operate(patternIn(:,sub));
                        try
                            activn((k-1)*sum(this.minMeanMax)+1:k*sum(this.minMeanMax),sub) = this.reduce(temproi);
                        catch
                            error(['was empty:' this.regions{k}.name]);
                        end
                    end
                    disp(k);
                end
            end
            patternOut = activn;
        end
        
        function reduced = reduce(this,roi)
            % Compute min/mean/max values
            roi = roi(~isnan(roi));
            reduced = nan(1,sum(this.minMeanMax));
            roi = sort(roi);
            k=1;
            if this.minMeanMax(1)
                reduced(k) = mean(roi(1:min(size(roi,1),this.n)));
                k = k + 1;
            end
            if this.minMeanMax(2)
                reduced(k) = mean(roi);
                k = k + 1;
            end
            if this.minMeanMax(3)
                reduced(k) = mean(roi(end-min(size(roi,1)-1,this.n-1):end));
                k = k + 1;
            end
        end
        
        function patternOut = reconstruct(this, patternIn)
            % compute graphical representation of feature weights
            k = 1;
            if this.minMeanMax(1)
                patMin = patternIn(k:sum(this.minMeanMax):end);
                [Bmin,IXmin] = sort(patMin);
                Cmin = cell(1,length(Bmin));
                for j = 1:length(Cmin)
                    Cmin{j} = this.regions{IXmin(j)}.maskName;
                end
                this.plotBarh(Bmin,Cmin,'min. peak rois')
                k = k +1;
            end
            
            if this.minMeanMax(2)
                patMean = patternIn(k:sum(this.minMeanMax):end);
                [Bmean,IXmean] = sort(patMean);
                Cmean = cell(1,length(Bmean));
                for j = 1:length(Cmean)
                    Cmean{j} = this.regions{IXmean(j)}.maskName;
                end
                this.plotBarh(Bmean,Cmean,'mean rois');
                k = k + 1;
            end
            
            if this.minMeanMax(3)
                patMax = patternIn(k:sum(this.minMeanMax):end);
                [Bmax,IXmax] = sort(patMax);
                Cmax = cell(1,length(Bmax));
                for j = 1:length(Cmax)
                    Cmax{j} = this.regions{IXmax(j)}.maskName;
                end
                this.plotBarh(Bmax,Cmax,'max. peak rois');
            end
        end
        
        function jpz = uiconfigure(this)
            jpz = uiconfigure@mania_pre_active_rois(this);
            quesMin  = spm_input('Min','+1','b','Yes*|No',[1 0],1);
            quesMean = spm_input('Mean','+1','b','Yes*|No',[1 0],1);
            quesMax  = spm_input('Max','+1','b','Yes*|No',[1 0],1);
            this.minMeanMax = [quesMin quesMean quesMax];
        end
        
        function new = copy(this)
            new = copy@mania_pre_active_rois(this);
            new.minMeanMax = this.minMeanMax;
        end
        
        function viewSettings(this)
            viewSettings@mania_pre_active_rois(this);
            optDesc = {'Minimum ', 'Mean ', 'Maximum'};
            opts = [optDesc{logical(this.minMeanMax)}];
            fprintf('  Conditions:\t%s\n',opts);
        end
        
    end
end