% Features Selection using Statistics Toolbox
classdef mania_pro_FeatureSelection < mania_processor & mania_statisticsWrapper
    
    properties
        direction = 'forward';           % direction foward / backward feature selection
        maxdev    = chi2inv(.95,1);      % Maximum Deviation value
        toleranceFunc = 'abs';           % tolerance function absolute or relative
        displayOpt = 'iter';
        crit     = @critfun;             % Criteria function
        mask     = [];                   % mask of selected features
    end
    
    methods (Static)
        function desc = description
            desc = 'Sequential Feature Selection using Matlab function sequentialfs';
        end
        function shortName = name
            shortName = 'Sequential Feature Selection';
        end
    end
    
    
    methods
        
        function bool = isConfigurable(this)
            bool = true;
        end
        
        function [train, test] = operate(this,train, test, trainLabels)
            % calls sequentialfs
            opt = statset('display',this.displayOpt,'TolFun',this.maxdev,'TolTypeFun',this.toleranceFunc);
            warning off all
            feat = sequentialfs(this.crit, train', trainLabels, 'cv', 'none', 'options', opt, 'direction', this.direction);
            warning on all
            this.mask = zeros(size(train,1),1);
            this.mask(feat) = 1;
            train = train(feat,:);
            test = test(feat,:);
        end
        
        function new = copy(this)
            new = copy@mania_processor(this);
            new.maxdev = this.maxdev;
            new.toleranceFunc = this.toleranceFunc;
            new.crit = this.crit;
            new.mask = this.mask;
            new.displayOpt = this.displayOpt;
            new.direction = this.direction;
        end
        
        function this =  uiconfigure(this)
            directionOpts = {'forward'; 'backward'};
            res = spm_input('Feature Selection Direction','+1','b','Forward*|Backward',[1 2],1)
            this.direction = directionOpts{res}
            res = spm_input('Citeria Function','+1','s','@critfun');
            this.crit = eval(res);
            tolOpts = {'abs'; 'rel'};
            res = spm_input('Tolerance Function','+1','b','absolute*|relative',[0 1],1);
            this.toleranceFunc = tolOpts{res+1};
            res = spm_input('Maximum Deviance','+1','s','chi2inv(.95,1)');
            this.maxdev = eval(res);
        end
        
        function viewSettings(this)
            viewSettings@mania_processor(this);
            fprintf('  direction:\t%s\n',this.direction);
            fprintf('  Tolerance function:\t%s\n',this.toleranceFunc);
            fprintf('  Max. Deviation:\t%d\n',this.maxdev);
            fprintf('  Crit. Function:\t%s\n',func2str(this.crit));
            
        end
        
        function patternOut = reoperate(this, patternIn)
            patternIn = reshape(patternIn,numel(patternIn),1);
            if size(patternIn,1) < size(this.mask,1) && ~mod(size(this.mask,1),size(patternIn,1))
                for k=1:size(preprop.mask,1)/size(patternIn,1)
                    patternIn = [patternIn; patternIn];
                end
            end
            patternOut = patternIn(this.mask,:);
        end
        
        function patternRecon  = reconstruct(this,patternIn)
            % everthing zero execept the features, that were choosen
            % everything else should get NaN or zero.
            patternRecon = zeros(size(this.mask,1),1);
            patternRecon(logical(this.mask)) = patternIn;
            patternRecon = patternRecon';
        end
        
    end
    
end