% Relieff algorithm using Matlab statistics toolbox
%
classdef mania_pro_Relieff < mania_processor
    
    properties
        relieffOpts = {};                   % algorithm options
        mask        = [];                   % mask of selected features
    end
    
    methods (Static)
        function desc = description
            desc = 'Feature selection using RELIEFF Algorithm (Matlab function)';
        end
        function shortName = name
            shortName = 'Relieff Algorithm';
        end
        function [state msg] = checkSetup
            [state msg] = mania_statisticsWrapper.checkSetup;
            if state
                % in older versions maybe not available
                state = (exist('relieff','file') == 2);
                if ~state
                    msg = 'Statistics Toolbox ok, but Relieff Algorithm is missing - maybe outdated matlab version';
                end
            end
        end
    end
    
    methods
        function bool = isConfigurable(this)
            bool = true;
        end
        
        function [train, test] = operate(this,train, test, trainLabels)
            % calls relieff
            if isempty(this.relieffOpts)
                throw(MException('mania:RelieffOpts', [this.name ': No options were specified for RELIEFF algorithm!']));
            end
            [ranked, weights] = relieff(train',trainLabels,this.relieffOpts{4},'method','classification');
            this.mask = zeros(size(train,1),1);
            this.mask(ranked(1:this.relieffOpts{2})) = 1;
            train = train(ranked(1:this.relieffOpts{2}),:);
            test = test(ranked(1:this.relieffOpts{2}),:);
        end
        
        function patternOut = reoperate(this, patternIn)
            patternIn = reshape(patternIn,numel(patternIn),1);
            if size(patternIn,1) < size(this.mask,1) && ~mod(size(this.mask,1),size(patternIn,1))
                for k=1:size(preprop.mask,1)/size(patternIn,1)
                    patternIn = [patternIn; patternIn];
                end
            end
            patternOut = patternIn(this.mask,:);
        end
        
        function patternRecon  = reconstruct(this,patternIn)
            % everthing zero execept the features, that were choosen
            patternRecon = zeros(size(this.mask,1),1);
            patternRecon(logical(this.mask)) = patternIn;
            patternRecon = patternRecon';
        end
        
        function jpz = uiconfigure(jpz)
            jpz.relieffOpts = {};
            repl=inputdlg({'Number',},'Number of features',1,{''});
            if ~isempty(repl{1})
                jpz.relieffOpts{length(jpz.relieffOpts)+1} = 'NUMBER';
                jpz.relieffOpts{length(jpz.relieffOpts)+1} = str2double(repl{1});
            end
            repl=inputdlg({'Number',},'K for K nearest Neighbours',1,{''});
            if ~isempty(repl{1})
                jpz.relieffOpts{length(jpz.relieffOpts)+1} = 'KNN';
                jpz.relieffOpts{length(jpz.relieffOpts)+1} = str2double(repl{1});
            end
        end
        
        function new = copy(this)
            new = copy@mania_processor(this);
            new.relieffOpts = this.relieffOpts;
            new.mask = this.mask;
        end
        
        function viewSettings(this)
            viewSettings@mania_processor(this);
            try
                fprintf('  Number of features:   %d\n',this.relieffOpts{2});
                fprintf('  k-nearest neighbours: %d\n',this.relieffOpts{4});
            catch
                fprintf('No settings specified\n');
            end
        end
        
    end
    
    
end