classdef mania_pro_fTestRanking < mania_processor
    
    properties
        mask;                                           % mask of ranked and choosen features
        cutoff = 0.05;               % how many feats
    end
    
    methods (Static)
        function desc = description
            desc = 'Rank features using F-Tests: ranks features according to p-values of F-Tests';
        end
        function shortName = name
            shortName = 'F-Test Feature Ranking';
        end
    end
    
    
    methods
        function bool = isConfigurable(this)
            bool = true;
        end
        
        function [train, test] = operate(this,train, test, trainLabels)
            % calls rankfeatures
            %ps = cellfun(@(x) vartestn(x',trainLabels','off'),num2cell(train,2));
            ps = cellfun(@(x) anova1(x,trainLabels','off'),num2cell(train,2));
            sorted = sort(ps);
            %cutoff = sorted(this.nFeats);
            cutoff = this.cutoff; %0.05;
            %fprintf('Cutoff:\t- %i\n',cutoff);
            ps(ps>cutoff) = nan;
            ps(~isnan(ps)) = true;
            ps(isnan(ps)) = false;
            fprintf('Num Feats:\t %i\n', sum(ps));
            %[feat,stat] = rankfeatures(train,trainLabels, this.rkopts{:});
            this.mask = zeros(size(train,1),1);
            this.mask(logical(ps)) = 1;
            train = train(logical(ps),:);
            test = test(logical(ps),:);
        end
        
        function patternOut = reoperate(this, patternIn)
            patternIn = reshape(patternIn,numel(patternIn),1);
            if size(patternIn,1) < size(this.mask,1) && ~mod(size(this.mask,1),size(patternIn,1))
                for k=1:size(preprop.mask,1)/size(patternIn,1)
                    patternIn = [patternIn; patternIn];
                end
            end
            patternOut = patternIn(this.mask,:);
        end
        
        function patternRecon  = reconstruct(this,patternIn)
            % everthing zero execept the features, that were choosen
            % everything else should get NaN or zero.
            patternRecon = zeros(size(this.mask,1),1);
            patternRecon(logical(this.mask)) = patternIn;
            patternRecon = patternRecon';
        end
        
        function this = uiconfigure(this)
            repl=inputdlg({'Number',},'p-value cutoff',1,{'0.05'});
            if ~isempty(repl{1})
                this.cutoff = str2double(repl{1});
            end
        end
        
        function new = copy(this)
            new = copy@mania_processor(this);
            new.cutoff = this.cutoff;
        end
        
        function viewSettings(this)
            viewSettings@mania_processor(this);
            fprintf('p-Value cutoff:\t- %i\n',this.cutoff);
        end
    end
    
    
    
    
end