%  Applies Principal Component Analysis on the training(!) patterns DURING
%  Cross-Validation.
%       uses Matlab's princomp with econ options!
%
classdef mania_pro_pca < mania_processor & mania_statisticsWrapper
    
   properties (SetAccess = private)
       pcaOpts = {'econ'};      % default option: econ
   end
   
   properties
      coeff;                    % computed coefficients
   end
   
       
    methods (Static)
        function desc = description
            desc = 'Principal Component Analysis (using Matlab function princomp) - takes only features with high Eigenvalues'; 
        end
        function shortName = name
            shortName = 'Principal Component analysis';
        end
    end
    
   methods
   
       function [train, test] = operate(this, train, test, trainLabels)
           % calls princomp
           [coeff score latent] = princomp(train', this.pcaOpts{:});
           this.coeff = coeff;
           train = train'*coeff; % = score // transform into pca space
           test = test'*coeff; % transform test samples in the same way
           train = train';
           test  = test';
       end
       
       function patternRecon  = reconstruct(this, patternIn)
           % reconstruct image
           patternRecon = patternIn / this.coeff;
           patternRecon = patternRecon';
       end
       
       function  patternOut = reoperate(this,patternIn)
          patternOut = patternIn' * this.coeff;
          patternOut = patternOut';
       end
       
       function viewSettings(this)
           viewSettings@mania_processor(this);
           fprintf('  Options: %s\n',this.pcaOpts{:});
           if ~isempty(this.coeff)
               fprintf('\tVoxels included: %i',size(this.coeff,2));
           end
       end
       
   end
end