% Abstract validator class
%   Just handles validation methods
classdef mania_validator < handle
    
    properties (SetAccess = protected)
        patternParser;              % patternParser, in general simpleParser
        rounds;                     % overall rounds, depends on validation method
        classifier;                 % the classifier
        roundPerformances = [];     % list of performances in the previous rounds
        predicted = [];             % list of predictions
        decFactors;                 % list of decision values
    end
    
    properties (GetAccess = public)
        classPerformance;           % the actual performance
        actualRound;                % the actual round (needed for waitbar)
        Start;                      % Start time (also waitbar)
        timer;                      % timer (tic), also waitbar
    end
    
    events
        cycle;                      % needed for waitbar, fire event after every round
    end
    
    methods
        
        function obj = mania_validator(classCollection,classifier,patternParser)
            % Constructor, just sets up properties and inits the parser
            obj.patternParser = patternParser;
            obj.patternParser.init;
            obj.classifier = classifier;
            obj.rounds = classCollection.patternCount;
        end
        
        function this = setClassifier(this, classifier)
            % set / change classifier
            this.classifier = classifier;
            this = this.patternParser.init;
        end
        
        function this = setClassCollection(this, classCollection)
            % set / change classCollection
            this.patternParser = simpleParser(classCollection);
            this = this.patternParser.init;
        end
        
        function rounds = getRoundCount(this)
            % returns the actual validation round
            rounds = this.rounds;
        end
        
        function this = setClassifierName(this,title)
            % set a name for classifier (useful for ensemble classifiers)
            this.classifier.title = title;
        end
        
        function pl = GetPerformanceList(this)
            % returns a list of performances
            pl = this.roundPerformances;
        end
        
        function perf = getPerformanceForSubject(this,testIdx)
            % return performance for a specific subject / leaveOut
            this = this.computeForSubjects(testIdx);
            perf = this.classPerformance;
        end
        
        function resetClassPerformance(this)
            % resets the performance, e.g. before beginning a new validation
            this.classPerformance.reset;
            this.classPerformance = classperf(this.patternParser.getLabels);
            this.predicted = [];
            this.decFactors = [];
        end
        
        function new = copy(this)
            pp_copy = this.patternParser.copy;
            new = feval(class(this),this.patternParser.classCollection,this.classifier,pp_copy);
            new.timer = this.timer;
            new.Start = this.Start;
            new.classPerformance = this.classPerformance;           % the actual performance
            new.actualRound = this.actualRound;                % the actual round (needed for waitbar)
            new.patternParser = pp_copy;              % patternParser, in general simpleParser
            new.rounds = this.rounds;                     % overall rounds, depends on validation method
            new.classifier = this.classifier;                 % the classifier
            new.roundPerformances = this.roundPerformances;     % list of performances in the previous rounds
            new.predicted = new.predicted;
        end
    end
    
    methods (Abstract)
        results = runValidation(this)            % run the validation, compute performance
        jv = computeForSubjects(testIdx);      % compute only performance for a specific subject.
    end
end