% to resize mask according to input patterns
%  without input: gui prompt will ask for input
% otherwise, images will be saved in temp system folder - input:
%   1: Original-Example-Image
%   2: Mask-File
%
function maskFName = resliceMask(varargin)
prefix = 'resized-';

% no input -> GUI
if ~nargin
    beta = spm_select(1,'image','Select Image which represents origin space.');
    mask = spm_select(inf,'image','Select Mask(s) to resize.');
else
    beta = varargin{1};
    origMask = varargin{2};
    [mdir mname mext] = fileparts(origMask);
    % if necessary, strip strange spm-ending
    mext = mext(1:4);
    origMask = [mdir filesep mname mext];
    mask = [tempdir filesep mname mext];
    copyfile(origMask,mask);
end

% setup SPM batch RESLICE
for mc=1:size(mask,1)
    tmp{mc}.spm.spatial.realign.write.data = {
        beta
        strtrim(mask(mc,:))
        };
    tmp{mc}.spm.spatial.realign.write.roptions.which = [1 0];
    tmp{mc}.spm.spatial.realign.write.roptions.interp = 4;
    tmp{mc}.spm.spatial.realign.write.roptions.wrap = [0 0 0];
    tmp{mc}.spm.spatial.realign.write.roptions.mask = 1;
    tmp{mc}.spm.spatial.realign.write.roptions.prefix = prefix;
end
isok = true;
% run batch
for k=1:size(tmp,2)
    matlabbatch{1} = tmp{k};
    out = spm_jobman('run',matlabbatch);
    isok = isok && (exist(out{1}.rfiles{1}(:,1:end-2),'file') ...
        || exist(out{1}.rfiles{1}(:,1:end),'file'));
end
if isok
    % for GUI
    if ~nargin
        h = msgbox(['New Mask has been saved as: ' out{1}.rfiles{1}(:,1:end-2)], 'Success','help');
        maskFName = out{1}.rfiles{1}(:,1:end-2);
    %  delete temporary copy of mask
    else
        maskFName = [tempdir filesep prefix mname mext];
        delete(mask);
    end
else
    errordlg('Mask could not be generated or saved!','Reslicing Failed');
end